==> aria2_1.21.0.orig.tar.gz: aria2-1.21.0/src/SftpDownloadCommand.cc <==
/* <!-- copyright */
/*
 * aria2 - The high speed download utility
 *
 * Copyright (C) 2015 Tatsuhiro Tsujikawa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */
/* copyright --> */
#include "SftpDownloadCommand.h"
#include "Request.h"
#include "SocketCore.h"
#include "Segment.h"
#include "DownloadEngine.h"
#include "RequestGroup.h"
#include "Option.h"
#include "FileEntry.h"
#include "SocketRecvBuffer.h"
#include "AuthConfig.h"
#include "SftpFinishDownloadCommand.h"

namespace aria2 {

SftpDownloadCommand::SftpDownloadCommand(
    cuid_t cuid, const std::shared_ptr<Request>& req,
    const std::shared_ptr<FileEntry>& fileEntry, RequestGroup* requestGroup,
    DownloadEngine* e, const std::shared_ptr<SocketCore>& socket,
    std::unique_ptr<AuthConfig> authConfig)
    : DownloadCommand(cuid, req, fileEntry, requestGroup, e, socket,
                      std::make_shared<SocketRecvBuffer>(socket)),
      authConfig_(std::move(authConfig))
{
  setWriteCheckSocket(getSocket());
}

SftpDownloadCommand::~SftpDownloadCommand() {}

bool SftpDownloadCommand::prepareForNextSegment()
{
  if (getOption()->getAsBool(PREF_FTP_REUSE_CONNECTION) &&
      getFileEntry()->gtoloff(getSegments().front()->getPositionToWrite()) ==
          getFileEntry()->getLength()) {

    auto c = make_unique<SftpFinishDownloadCommand>(
        getCuid(), getRequest(), getFileEntry(), getRequestGroup(),
        getDownloadEngine(), getSocket());

    c->setStatus(Command::STATUS_ONESHOT_REALTIME);
    getDownloadEngine()->setNoWait(true);
    getDownloadEngine()->addCommand(std::move(c));

    if (getRequestGroup()->downloadFinished()) {
      // To run checksum checking, we had to call following function here.
      DownloadCommand::prepareForNextSegment();
    }
    return true;
  }

  auto rv = DownloadCommand::prepareForNextSegment();
  if (rv) {
    return true;
  }
  // sftp may not get incoming data.  Enable write check to make this
  // command invoke.
  setWriteCheckSocket(getSocket());
  return false;
}

int64_t SftpDownloadCommand::getRequestEndOffset() const
{
  return getFileEntry()->getLength();
}

bool SftpDownloadCommand::shouldEnableWriteCheck()
{
  return getSocket()->wantWrite() || !getSocket()->wantRead();
}

} // namespace aria2

==> aria2_1.36.0.orig.tar.gz: aria2-1.36.0/src/SftpDownloadCommand.cc <==
/* <!-- copyright */
/*
 * aria2 - The high speed download utility
 *
 * Copyright (C) 2015 Tatsuhiro Tsujikawa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */
/* copyright --> */
#include "SftpDownloadCommand.h"
#include "Request.h"
#include "SocketCore.h"
#include "Segment.h"
#include "DownloadEngine.h"
#include "RequestGroup.h"
#include "Option.h"
#include "FileEntry.h"
#include "SocketRecvBuffer.h"
#include "AuthConfig.h"
#include "SftpFinishDownloadCommand.h"

namespace aria2 {

SftpDownloadCommand::SftpDownloadCommand(
    cuid_t cuid, const std::shared_ptr<Request>& req,
    const std::shared_ptr<FileEntry>& fileEntry, RequestGroup* requestGroup,
    DownloadEngine* e, const std::shared_ptr<SocketCore>& socket,
    std::unique_ptr<AuthConfig> authConfig)
    : DownloadCommand(cuid, req, fileEntry, requestGroup, e, socket,
                      std::make_shared<SocketRecvBuffer>(socket)),
      authConfig_(std::move(authConfig))
{
  setWriteCheckSocket(getSocket());
}

SftpDownloadCommand::~SftpDownloadCommand() = default;

bool SftpDownloadCommand::prepareForNextSegment()
{
  if (getOption()->getAsBool(PREF_FTP_REUSE_CONNECTION) &&
      getFileEntry()->gtoloff(getSegments().front()->getPositionToWrite()) ==
          getFileEntry()->getLength()) {

    auto c = make_unique<SftpFinishDownloadCommand>(
        getCuid(), getRequest(), getFileEntry(), getRequestGroup(),
        getDownloadEngine(), getSocket());

    c->setStatus(Command::STATUS_ONESHOT_REALTIME);
    getDownloadEngine()->setNoWait(true);
    getDownloadEngine()->addCommand(std::move(c));

    if (getRequestGroup()->downloadFinished()) {
      // To run checksum checking, we had to call following function here.
      DownloadCommand::prepareForNextSegment();
    }
    return true;
  }

  auto rv = DownloadCommand::prepareForNextSegment();
  if (rv) {
    return true;
  }
  // sftp may not get incoming data.  Enable write check to make this
  // command invoke.
  setWriteCheckSocket(getSocket());
  return false;
}

int64_t SftpDownloadCommand::getRequestEndOffset() const
{
  return getFileEntry()->getLength();
}

bool SftpDownloadCommand::shouldEnableWriteCheck()
{
  return getSocket()->wantWrite() || !getSocket()->wantRead();
}

} // namespace aria2
